#include "GBK2Unicode.h"

// compare two GBK strings by Unicode codes
// Note: this function returns 0 for invalid GBK strings!
int strcmp_Unicode_GBK(char *string1, char *string2)
{
	unsigned char bGBKCode1[2], bGBKCode2[2];
	unsigned short index1, unicode1, index2, unicode2;
	while(*string1!='\0' && *string2!='\0')
	{
		bGBKCode1[1] = *string1++;
		bGBKCode2[1] = *string2++;
			if(*string1=='\0' || *string2=='\0') return 0;
		bGBKCode1[0] = *string1++;
		bGBKCode2[0] = *string2++;
		index1 = (bGBKCode1[1]-0x81)*192 + (bGBKCode1[0]-0x40);
			if(index1<0 || index1>=GBK_CODES_NUMBER2) return 0;
		unicode1 = gbk2uni[index1];
			if(unicode1 == 0) return 0;
		index2 = (bGBKCode2[1]-0x81)*192 + (bGBKCode2[0]-0x40);
			if(index2<0 || index2>=GBK_CODES_NUMBER2) return 0;
		unicode2 = gbk2uni[index2];
			if(unicode2 == 0) return 0;
		if(unicode1 > unicode2) return 1;
		if(unicode1 < unicode2) return -1;
	}
	if(*string1 != '\0') return 1; // strlen(string1) > strlen(string2)
	if(*string2 != '\0') return -1; // strlen(string1) < strlen(string2)
	return 0; // strlen(string1) = strlen(string2)
}
// compare two strings by Unicode
int strcmp_Unicode(char *string1, char *string2, int index1, int index2)
{
	unsigned char bGBK1 = 1, bGBK2 = 1;
	int rtn;
	char *p1, *p2;
	while(*string1!='\0' && *string2!='\0')
	{
		bGBK1 = IsGBKChar(string1);
		bGBK2 = IsGBKChar(string2);
		if((bGBK1 && bGBK2) || (!bGBK1 && !bGBK2))// two GBK pinyin (or two non-GBK) strings
		{
			p1 = strchr(string1, SEPARATOR_IN_NAME);
			if(p1) *p1 = '\0';
			p2 = strchr(string2, SEPARATOR_IN_NAME);
			if(p2) *p2 = '\0';
			if(bGBK1 && bGBK2) rtn = strcmp_Unicode_GBK(string1, string2);
			else // !bGBK1 && !bGBK2
			{
				if(stricmp(string1, string2)) // only return when string /neq string2
				{
					if(index1 > index2) rtn = 1;
					else if(index1 < index2) rtn = -1;
					else rtn = 0;
				}
				else rtn = 0;
			}
			if(p1) *p1 = SEPARATOR_IN_NAME;
			if(p2) *p2 = SEPARATOR_IN_NAME; // restore the strings before returning
			if(rtn) return rtn;
			if(p1) string1 = p1+1;
			else string1 += strlen(string1); // end of string1
			if(p2) string2 = p2+1;
			else string2 += strlen(string2); // end of string2
		}
		else if(bGBK1) // && !bGBK2
		{
			switch(bGBK2NonGBK)
			{
			case GBK_LESS_NON_GBK: return -1;
			// case NON_GBK_LESS_GBK:
			default: return 1;
			}
		}
		else // bGBK2 && !bGBK1
		{
			switch(bGBK2NonGBK)
			{
			case GBK_LESS_NON_GBK: return 1;
			// case NON_GBK_LESS_GBK:
			default: return -1;
			}
		}
	}
	if(*string1 != '\0') return 1;
	if(*string2 != '\0') return -1;
	return 0;
}
int strcmp_Unicode_Authors(char *szAuthors1, char *szAuthors2, int index1, int index2)
{
	int rtn;
	while(*szAuthors1!='\0' && *szAuthors1!='\0')
	{
		rtn = strcmp_Unicode(szAuthors1, szAuthors2, index1, index2);
		if(rtn != 0) return rtn;
		szAuthors1 += strlen(szAuthors1)+1;
		szAuthors2 += strlen(szAuthors2)+1;
	}
	if(*szAuthors1 != '\0') return 1;
	if(*szAuthors2 != '\0') return -1;
	return 0;
}

/*
//////////////////////////////////////////////////////////////////

struct GBK2UnicodeINFO {
	char strGBKCodes[2];//GBK
	char strUnicode[5];//Unicode
} *pGBK2UnicodeInfo = NULL;
//int nGBKCodes;

//GBK2Uni.infļȡUnicodeϢ
int getGBK2UnicodeInfo()
{
	int nLine;
	char strInfo[_LINE_MAX], *p;
	FILE *fileGBK2Unicode;

	fileGBK2Unicode = OpenFileInPATH("GBK2Uni.inf", "rt");
	if(fileGBK2Unicode == NULL) {
		fprintf(stderr, "Failed to open GBK2Uni.inf!\n");
		return -1;
	}

	nGBKCodes = (0xFE - 0x80) * GBKperLB;
	pGBK2UnicodeInfo = (struct GBK2UnicodeINFO*) calloc(nGBKCodes, sizeof(struct GBK2UnicodeINFO));
	if( pGBK2UnicodeInfo == NULL) {
		fprintf(stderr, "System memory is not enough to allocate %d bytes!\n", nGBKCodes * sizeof(struct GBK2UnicodeINFO));
		return -1;
	}

	nLine = 0;
	while(!feof(fileGBK2Unicode)) {
		if(fgets(strInfo, _LINE_MAX, fileGBK2Unicode) == NULL) break;
		if(!strncmp(strInfo, "0x", 2)) {
			p = strstr(strInfo + 2, "0x");
			if(p == NULL) continue;
			i2cstr(nLine, pGBK2UnicodeInfo[nLine].strGBKCodes);
			strncpy(pGBK2UnicodeInfo[nLine].strUnicode, p + 2, 4);
			nLine ++;
		}
	}
	fclose(fileGBK2Unicode);
	return 1;
}

//ѡ, , and , editors. and .бתΪ׼Unicodeַ
int Authors2Unicode(char *strAuthors, char *strUnicode)
{
	char *strTemp, *strTemp2;
	BYTE bHigh, bLow;
	char *_strAuthors;
	_strAuthors = strAuthors;

	strTemp = calloc(strlen(strAuthors)+1, 1);
	if(strTemp == NULL) return -1;
	strTemp2 = strTemp;
	//бתΪָʽԷƴַȽ
	while(*_strAuthors!='\0') {
		if(*_strAuthors == '\n' || *_strAuthors == '\r') break;
		if(!strnicmp(_strAuthors, ", and ", 6)) {//replacing ', and ' with NAME_SEPARATOR
			*strTemp2 = NAME_SEPARATOR; strTemp2 ++; 
			_strAuthors += 6; continue;
		}
		else if(!strnicmp(_strAuthors, ", editors", 9)) break;//discarding the left content
		else if(!strnicmp(_strAuthors, ", ", 2)) {//replacing ', ' with NAME_SEPARATOR
			*strTemp2 = NAME_SEPARATOR; strTemp2 ++; 
			_strAuthors += 2; continue;
		}
		else if(!strnicmp(_strAuthors, " and ", 5)) {//replacing ' and ' with NAME_SEPARATOR
			*strTemp2 = NAME_SEPARATOR; strTemp2 ++; 
			_strAuthors += 5; continue;
		}
		else {
			if(strTemp2 > strTemp) {//add an Before_NonChineseName for an non-Chinese author
				if(*(strTemp2-1) == NAME_SEPARATOR && !GBK_HIGH(*_strAuthors)) *strTemp2++=Before_NonChineseName;
			}
			else if((strTemp2 == strTemp) && !GBK_HIGH(*_strAuthors)) *strTemp2++=Before_NonChineseName;
			*strTemp2++=*_strAuthors++;
		}
	}
	if(*(strTemp2-1) == '.') strTemp2 --;
	*strTemp2 = '\0'; strTemp2 = strTemp;

	while(*strTemp2!='\0') {
		bHigh = (BYTE)*strTemp2++;
		if(!GBK_HIGH(bHigh)) {
			*strUnicode++ = bHigh;
			continue;
		}
		bLow = (BYTE)*strTemp2;
		if(!GBK_LOW(bLow)) {//skip bHigh
			fprintf(stderr, "Error: wrong GBK character %x%x is found!\n", bHigh, bLow);
			continue;
		}
		strTemp2++;
		strcpy(strUnicode, pGBK2UnicodeInfo[cbytes2i(bHigh, bLow)].strUnicode);
		while(*strUnicode!='\0') strUnicode++;//go to end of strUnicode
	}
	*strTemp2 = '\0';

	free(strTemp);
	return 1;
}
*/