struct GBK2pyINFO {
	char strGBKCodes[2];//GBK
	char strPinyin[PINYIN_MAX];//ƴ
	char strPinyin2[PINYIN_MAX];//ʱƴ
} *pGBK2pyInfo = NULL;
struct DoubleName2pyINFO {
	char strDoubleName[4]; //ָ
	char strPinyin[2][PINYIN_MAX]; // ֺ͵ڶֵƴ
} *pDoubleName2pyInfo = NULL;
int nGBKCodes, nDoubleNames;

//GBK2py.infļȡƴϢ
int getGBK2pyInfo()
{
	int nLine, bDoubleName;
	char strInfo[LINE_MAX];
	FILE *fileGBK2py;

	fileGBK2py = OpenFileInPATH("GBK2py.inf", "rt");
	if(fileGBK2py == NULL) {
		fprintf(stderr, "Failed to open GBK2py.inf!\n");
		return -1;
	}

	nGBKCodes = (0xFE - 0x80) * GBKperLB;
	pGBK2pyInfo = (struct GBK2pyINFO*) calloc(nGBKCodes, sizeof(struct GBK2pyINFO));
	if( pGBK2pyInfo == NULL) {
		fprintf(stderr, "System memory is not enough to allocate %d bytes!\n", nGBKCodes * sizeof(struct GBK2pyINFO));
		return -1;
	}

	nLine = 0; bDoubleName = 1;
	while(!feof(fileGBK2py)) {
		if(fgets(strInfo, LINE_MAX, fileGBK2py) == NULL) break;
		if(strInfo[0] == '%' || strInfo[0] == '\n' || strInfo[0] == '\r' ) continue;
		if(strstr(strInfo, "[⸴=") == strInfo) {
			nLine = 0; bDoubleName = 1;
			nDoubleNames = atoi(strInfo+10);
			pDoubleName2pyInfo = (struct DoubleName2pyINFO*) calloc(nDoubleNames, sizeof(struct DoubleName2pyINFO));
			if( pDoubleName2pyInfo == NULL) {
				fprintf(stderr, "System memory is not enough to allocate %d bytes!\n", nDoubleNames * sizeof(struct DoubleName2pyINFO));
				return -1;
			}
		}
		else if(strstr(strInfo, "[GBK=") == strInfo) {
			nLine = 0; bDoubleName = 0;
		}
		else {
			if(bDoubleName)
				sscanf(strInfo, "%s %s %s", pDoubleName2pyInfo[nLine].strDoubleName, 
					pDoubleName2pyInfo[nLine].strPinyin[0], pDoubleName2pyInfo[nLine].strPinyin[1]);
			else
				sscanf(strInfo, "%c%c %s %s", &(pGBK2pyInfo[nLine].strGBKCodes[0]), 
					&(pGBK2pyInfo[nLine].strGBKCodes[1]), pGBK2pyInfo[nLine].strPinyin, 
					pGBK2pyInfo[nLine].strPinyin2);
			nLine ++;
		}
	}
	fclose(fileGBK2py);
	return 1;
}

// transform a single GBK string to multiple pinyin strings
// "San" => "zhang1\asan1\aSan\0".
void String2Py(char *szPy, char *szSour)
{
	int i, bFamilyName = 1;
	if(*szSour == '\0') return;
	while(*szSour=='{' || *szSour==' ' || *szSour==',') szSour++; // skip continuous spaces
	i = strlen(szSour) - 1;
	if(szSour[i] == '}') szSour[i] = '\0'; // remove the last '}' (if any)
	while(*szSour != '\0')
	{
		while(*szSour==' ' || *szSour==',') szSour++; // skip continuous spaces
		if(*szSour == '\0') break;
		if(IsGBKChar(szSour))
		{
			if(bFamilyName)
			{
				for(i=0; i<nDoubleNames; i++)
				{
					if(!strncmp(szSour, pDoubleName2pyInfo[i].strDoubleName, 4))
					{
						*szPy++ = *szSour++;
						*szPy++ = *szSour++; // copy the first GBK character
						strcpy(szPy, pDoubleName2pyInfo[i].strPinyin[0]);
						szPy += strlen(szPy);
						*szPy++ = SEPARATOR_IN_NAME;
						*szPy++ = *szSour++;
						*szPy++ = *szSour++; // copy the second GBK character
						strcpy(szPy, pDoubleName2pyInfo[i].strPinyin[1]);
						szPy += strlen(szPy);
						*szPy++ = SEPARATOR_IN_NAME;
						bFamilyName = 0;
						break;
					}
				}
				if(!bFamilyName) continue;
			}
			if(!strnicmp(szSour+2, "\\pinyin{", 8))
			{
				*szPy++ = *szSour++;
				*szPy++ = *szSour++; // copy the GBK character
				szSour += 8;
				while(*szSour!='}' && *szSour!='\0') *szPy++ = *szSour++;
				if(*szSour == '}') szSour++;
			}
			else
			{
				i = cstr2i(szSour);
				*szPy++ = *szSour++;
				*szPy++ = *szSour++; // copy the GBK character
				if(bFamilyName)
				{
					strcpy(szPy, pGBK2pyInfo[i].strPinyin2);
					bFamilyName = 0;
				}
				else strcpy(szPy, pGBK2pyInfo[i].strPinyin);
				szPy += strlen(szPy);
			}
			*szPy++ = SEPARATOR_IN_NAME;
		}
		else
		{
			// *szPy++ = ENGLISH_PREFIX;
			while(1)
			{
				*szPy++ = *szSour++;
				if(*szSour=='\0' || IsGBKChar(szSour)) break;
				if(*szSour==' ' || *szSour==',') break;
			}
			*szPy++ = SEPARATOR_IN_NAME;
		}
	}
	szPy[-1] = '\0';
}
/*
void Authors2Py(char *strPinyin, char *strAuthors)
{
	while(*strAuthors != '\0')
	{
		GBK2Py(strPinyin, strAuthors);
		strPinyin += strlen(strPinyin)+1;
		strAuthors += strlen(strAuthors)+1;
	}
	*strPinyin = '\0';
}*/

// compare two pinyin strings (may contain both GBK pinyin-s and non-GBK characters)
int strcmp_pinyin(char *string1, char *string2, int index1, int index2)
{
	unsigned char bGBK1 = 1, bGBK2 = 1;
	int rtn;
	char *p1, *p2;
	while(*string1!='\0' && *string2!='\0')
	{
		bGBK1 = IsGBKChar(string1);
		bGBK2 = IsGBKChar(string2);
		if((bGBK1 && bGBK2) || (!bGBK1 && !bGBK2))// two GBK pinyin (or two non-GBK) strings
		{
			p1 = strchr(string1, SEPARATOR_IN_NAME);
			if(p1) *p1 = '\0';
			p2 = strchr(string2, SEPARATOR_IN_NAME);
			if(p2) *p2 = '\0';
			if(bGBK1 && bGBK2)
			{
				rtn = stricmp(string1+2, string2+2);
				if(rtn == 0) // if pinyin1 = pinyin2, turn to compare the original GBK characters
				{
					if(string1[0] > string2[0]) rtn = 1;
					else if(string1[0] < string2[0]) rtn = 1;
					else if(string1[1] > string2[1]) rtn = 1;
					else if(string1[1] < string2[1]) rtn = 1;
					else rtn = 0;
				}
			}
			else // !bGBK1 && !bGBK2
			{
				if(stricmp(string1, string2)) // only return when string /neq string2
				{
					if(index1 > index2) rtn = 1;
					else if(index1 < index2) rtn = -1;
					else rtn = 0;
				}
				else rtn = 0;
			}
			if(p1) *p1 = SEPARATOR_IN_NAME;
			if(p2) *p2 = SEPARATOR_IN_NAME;
			if(rtn != 0) return rtn;
			if(p1) string1 = p1+1;
			else string1 += strlen(string1); // end of string1
			if(p2) string2 = p2+1;
			else string2 += strlen(string2); // end of string2
		}
		else if(bGBK1) // && !bGBK2
		{
			switch(bGBK2NonGBK)
			{
			case GBK_LESS_NON_GBK: return -1;
			// case NON_GBK_LESS_GBK:
			default: return 1;
			}
		}
		else // bGBK2 && !bGBK1
		{
			switch(bGBK2NonGBK)
			{
			case GBK_LESS_NON_GBK: return 1;
			// case NON_GBK_LESS_GBK:
			default: return -1;
			}
		}
	}
	if(*string1 != '\0') return 1;
	if(*string2 != '\0') return -1;
	return 0;
}
int strcmp_pinyin_Authors(char *szAuthors1, char *szAuthors2, int index1, int index2)
{
	int rtn;
	while(*szAuthors1!='\0' && *szAuthors1!='\0')
	{
		rtn = strcmp_pinyin(szAuthors1, szAuthors2, index1, index2);
		if(rtn != 0) return rtn;
		szAuthors1 += strlen(szAuthors1)+1;
		szAuthors2 += strlen(szAuthors2)+1;
	}
	if(*szAuthors1 != '\0') return 1;
	if(*szAuthors2 != '\0') return -1;
	return 0;
}

/*

//ѡ, , and , editors. and .бתΪ׼ƴַ
//עһǳĵ⣺isalphadebug汾λrelease汾򲻼λ
//debugrelease汾ڴӢĲʱִ
//е!GBK_HIGHisalpha棡
int Authors2py(char *strAuthors, char *strPinyin)
{
	char *strTemp, *strTemp2;
	BYTE bHigh, bLow, bFamilyName;
	int i, k;
	char *_strAuthors;
	_strAuthors = strAuthors;

	strTemp = calloc(strlen(strAuthors)+1, 1);
	if(strTemp == NULL) return -1;
	strTemp2 = strTemp;
	//бתΪָʽԷƴַȽ
	while(*_strAuthors!='\0') {
		if(*_strAuthors == '\n' || *_strAuthors == '\r') break;
		if(!strnicmp(_strAuthors, ", and ", 6)) {//replacing ', and ' with NAME_SEPARATOR
			*strTemp2 = NAME_SEPARATOR; strTemp2 ++; 
			_strAuthors += 6; continue;
		}
		else if(!strnicmp(_strAuthors, ", editors", 9)) break;//discarding the left content
		else if(!strnicmp(_strAuthors, ", ", 2)) {//replacing ', ' with NAME_SEPARATOR
			*strTemp2 = NAME_SEPARATOR; strTemp2 ++; 
			_strAuthors += 2; continue;
		}
		else if(!strnicmp(_strAuthors, " and ", 5)) {//replacing ' and ' with NAME_SEPARATOR
			*strTemp2 = NAME_SEPARATOR; strTemp2 ++; 
			_strAuthors += 5; continue;
		}
		else {
			if(strTemp2 > strTemp) {//add an Before_NonChineseName for an non-Chinese author
				if(*(strTemp2-1) == NAME_SEPARATOR && !GBK_HIGH(*_strAuthors)) *strTemp2++=Before_NonChineseName;
			}
			else if((strTemp2 == strTemp) && !GBK_HIGH(*_strAuthors)) *strTemp2++=Before_NonChineseName;
			*strTemp2++=*_strAuthors++;
		}
	}
	if(*(strTemp2-1) == '.') strTemp2 --;
	*strTemp2 = '\0'; strTemp2 = strTemp;

	bFamilyName = 1;
	while(*strTemp2!='\0') {
		if(*strTemp2 == '\n' || *strTemp2 == '\r') break;
		if(GBK_HIGH(*strTemp2)) {
			//⸴յĻ
			if(bFamilyName) {
				for(k=0;k<nDoubleNames;k++) {
					if(!strncmp(strTemp2, pDoubleName2pyInfo[k].strDoubleName, 4)) {
						strcpy(strPinyin, pDoubleName2pyInfo[k].strPinyin[0]);
						strcat(strPinyin, pDoubleName2pyInfo[k].strPinyin[1]);
						while(*strPinyin!='\0') strPinyin++;//go to end of strPinyin
						strTemp2 += 4; break;
					}
				}
				bFamilyName = 0;
			}
			bHigh = (BYTE)*strTemp2++;
			bLow = (BYTE)*strTemp2;
			if(!GBK_LOW(bLow)) {//skip bHigh
				fprintf(stderr, "Error: wrong GBK character %x%x is found!\n", bHigh, bLow);
				continue;
			}
			strTemp2 ++;
			//ֱֵƴ
			if(!strncmp(strTemp2, "\\pinyin", 7)) {
				strTemp2 += 7;
				if(*strTemp2 == '{') strTemp2++;
				else fprintf(stderr, "Error: missing '{' in \\pinyin command!\n");
				while(isalpha(*strTemp2) || isdigit(*strTemp2)) {
					*strPinyin++=*strTemp2++;
				}
				if(*strTemp2 == '}') strTemp2++;
				else fprintf(stderr, "Error: missing '}' in \\pinyin command!\n");
			}
			//GBK2py.infļȡǰֵĳƴ
			else {
				i = cbytes2i(bHigh, bLow);
				if( i == -1) {
					fprintf(stderr, "Error: wrong GBK character %x%x is found!\n", bHigh, bLow);
					continue;
				}
				//ͨյĻ
				if(bFamilyName) {
					strcpy(strPinyin, pGBK2pyInfo[i].strPinyin2);
					bFamilyName = 0;
				}
				//е
				else strcpy(strPinyin, pGBK2pyInfo[i].strPinyin);
				while(*strPinyin!='\0') strPinyin++;//go to end of strPinyin
			}
		}
		else {
			if(*strTemp2 == NAME_SEPARATOR) bFamilyName = 1;//֮ļ
			*strPinyin++=*strTemp2++;
		}
	}
	*strTemp2 = '\0';

	free(strTemp);
	return 1;
}
*/