#include "GBKcommon.c"
#include "GBK2pinyin.c"
#include "GBK2Unicode.c"
#include "GBK2strokes.c"

int strcmp_Authors(char *szAuthors1, char *szAuthors2, int index1, int index2)
{
	switch(bSort)
	{
	case SORT_BY_PINYIN:
		return strcmp_pinyin_Authors(szAuthors1, szAuthors2, index1, index2);
	case SORT_BY_STROKES_O:
	case SORT_BY_STROKES_N:
		return strcmp_strokes_Authors(szAuthors1, szAuthors2, index1, index2);
	case SORT_BY_UNICODE:
		return strcmp_Unicode_Authors(szAuthors1, szAuthors2, index1, index2);
	default: // determined only by the values of index1 and index2
		if(index1 > index2) return 1;
		if(index1 < index2) return 1;
		return 0;
	}
}

struct BIBITEM_INFO {
	char	strAuthors[AUTHORS_MAX]; //бЧַ (ƴ׻ʻ)
	int		nStartPos; //bblļеʼλ
	BYTE	nLine; //bblļе, <256
	int		nLastBibitem; //ָһĿ̬-1൱ڶ̬еĿָNULL
	int		nNextBibitem; //ָһĿ̬-1൱ڶ̬еĿָNULL
} *pBibitemInfo = NULL;
int nBibitems; //Ŀ
int nFirstBibitem; //һĿ: [0, nBibitems-1]
int nLastBibitem; //һĿ: [0, nBibitems-1]

#define BEGIN_THEBIBLIOGRAPHY	"\\begin{thebibliography}{"
#define END_THEBIBLIOGRAPHY		"\\end{thebibliography}"

/*
int __atoi(char *string)
{
	int digit_size = 0, value, i;
	while(isdigit(*string)) digit_size++;
	value = 1;
	for(i=0; i<digit_size; i++) value *= 10;
	return value-1; // 10^{digit_size}-1
} */

int getBibitemsNumber()
{
	int i = 0;
	while(!feof(Fin))
	{
		if(fgets(b_in, nBibFileSize, Fin) == NULL) break;
		if(strstr(b_in, BIBPREFIX)) i++;
	}
	rewind(Fin);
	return i;
}

int getBibitems()
{
	int i = -1, j, nPos, nLastLine;
	nLine = 0;
	nBibitems = getBibitemsNumber();
	while(!feof(Fin))
	{
		nPos = ftell(Fin);
		if(fgets(b_in, nBibFileSize, Fin) == NULL) break;
		nLine++;
		b2_in = b_in;
		p = strstr(b2_in, BEGIN_THEBIBLIOGRAPHY);
		if(p != NULL)
		{
			// nBibitems = __atoi(p+strlen(BEGIN_THEBIBLIOGRAPHY));
			pBibitemInfo = (struct BIBITEM_INFO*)calloc(nBibitems, sizeof(struct BIBITEM_INFO));
			if(pBibitemInfo == NULL)
			{
				fprintf(stderr, "System memory is not enough to allocate %d bytes!\n", nBibitems * sizeof(struct BIBITEM_INFO));
				return -1;
			}
			for(j = 0; j < nBibitems; j++)
			{
				pBibitemInfo[j].nLastBibitem = pBibitemInfo[j].nNextBibitem = -1;
			}
			continue;
		}
		p = strstr(b2_in, END_THEBIBLIOGRAPHY);
		if(p != NULL)
		{
			pBibitemInfo[i].nLine = nLine - nLastLine;
			continue;
		}
		p = strstr(b2_in, BIBPREFIX);
		if(p != NULL)
		{
			if(i >= 0) pBibitemInfo[i].nLine = nLine - nLastLine;
			i++; nLastLine = nLine;
			pBibitemInfo[i].nStartPos = nPos;
			b2_in = p + strlen(BIBPREFIX);//skip '\bibitem'
			if(*b2_in == '[')
			{
				if(strchr(b2_in, ']')) bItemDataType = LINE_AUTHORS;
				else bItemDataType = LINE_BIBITEM;
			}
			else if(*b2_in == '{') bItemDataType = LINE_AUTHORS;
			continue;
		}
		if(bItemDataType == LINE_BIBITEM) {//find the end ']'
			if(strchr(b2_in, ']')) bItemDataType = LINE_AUTHORS;
		}
		else if(bItemDataType == LINE_AUTHORS) {
			ProcessAuthors(pBibitemInfo[i].strAuthors, b2_in);
			bItemDataType = LINE_EMPTY;
		}
	}
	return 0;
}

void freeall()
{
	if(b_in != NULL) free(b_in);
	if(pGBK2pyInfo != NULL) free(pGBK2pyInfo);
	if(pDoubleName2pyInfo != NULL) free(pDoubleName2pyInfo);
	if(pBibitemInfo != NULL) free(pBibitemInfo);
}

void sortBibitems()
{
	int i, j;
	nFirstBibitem = nLastBibitem = 0; // assign the first item as the only node
	for(i=1; i<nBibitems; i++)
	{
		for(j=nFirstBibitem; j!=-1; j=pBibitemInfo[j].nNextBibitem)
		{
			//try to find the first item such that it's strAuthors > pBibitemInfo[i].strAuthors
			if(strcmp_Authors(pBibitemInfo[j].strAuthors, pBibitemInfo[i].strAuthors, j, i) > 0)
			{
				if(j != nFirstBibitem)
				{
					pBibitemInfo[pBibitemInfo[j].nLastBibitem].nNextBibitem = i;
					pBibitemInfo[i].nLastBibitem = pBibitemInfo[j].nLastBibitem;
				}
				else
				{
					nFirstBibitem = i;
					pBibitemInfo[i].nLastBibitem = -1;
				}
				pBibitemInfo[i].nNextBibitem = j;
				pBibitemInfo[j].nLastBibitem = i;
				break;
			}
		}
		if(j == -1) // not found
		{
			pBibitemInfo[nLastBibitem].nNextBibitem = i;
			pBibitemInfo[i].nLastBibitem = nLastBibitem;
			pBibitemInfo[i].nNextBibitem = -1;
			nLastBibitem = i;
		}
	}
}

int SortBibitems()
{
	int i, j;
	if(bSort == SORT_BY_PINYIN)
	{
		if(getGBK2pyInfo() == -1) return -1;
	}
	Editors2EditorsShort(str_Editors2Char_short, str_Editors2Char);
	Editors2EditorsShort(str_Editor2Char_short, str_Editor2Char);
	getBibitems();
	sortBibitems();

	//generate re-sorted bbl file
	rewind(Fin);
	while(ftell(Fin) < pBibitemInfo[0].nStartPos)
	{
		if(fgets(b_in, nBibFileSize, Fin) == NULL) break;
		fputs(b_in, Fout);
	}

	for(i=nFirstBibitem; i!=-1; i=pBibitemInfo[i].nNextBibitem)
	{
		fseek(Fin, pBibitemInfo[i].nStartPos, SEEK_SET);
		for(j=0; j<pBibitemInfo[i].nLine; j++)
		{
			if(fgets(b_in, nBibFileSize, Fin) == NULL) break;
			fputs(b_in, Fout);
		}
	}
	fprintf(Fout, "%s\n", END_THEBIBLIOGRAPHY);
	
	fclose(Fin);
	fclose(Fout);
	sprintf(bakname, "%s.bak",inname);
	remove(bakname);
	rename(inname,bakname);
	rename(outname,inname);

	Fin = fopen(inname, "r");
	if(!Fin)
	{
		fprintf(stderr, "Failed to open %s for reading!\n", inname);
		freeall(); return -1;
	}
	Fout = fopen(outname, "wt");
	if(!Fout)
	{
		fprintf(stderr, "Cannot open %s for writing!\n", outname);
		freeall(); fclose(Fin); return -1;
	}
	return 0;
}

/*
int sortBibitems()
{
	//sort Chinese bibitems and skip all English bibitems
	int  i, index, nFirstEnglishBibitem = -1;
	nFirstBibitem = nLastBibitem = 0;
	for(i = 1; i < nBibitems; i++) {
		if(pBibitemInfo[i].strAuthors[0] == Before_NonChineseName) {
			if(nFirstEnglishBibitem == -1) nFirstEnglishBibitem = i;
			continue;
		}
		index = nFirstBibitem;
		// 㷨ôʵֵģ
		// debugһ¿ô¡
		for(; index != -1; index = pBibitemInfo[index].nNextBibitem) {//뵱ǰڵ
			if(index == i) continue;//ǰĿ
			if(strcmp(pBibitemInfo[index].strAuthors, pBibitemInfo[i].strAuthors) > 0) {
				//ҵһpBibitemInfo[i]Ľڵ
				if(index != nFirstBibitem) {
					pBibitemInfo[pBibitemInfo[index].nLastBibitem].nNextBibitem = i;
					pBibitemInfo[i].nLastBibitem = pBibitemInfo[index].nLastBibitem;
				}
				else {
					nFirstBibitem = i;
					pBibitemInfo[i].nLastBibitem = -1;
				}
				pBibitemInfo[i].nNextBibitem = index;
				pBibitemInfo[index].nLastBibitem = i;
				break;
			}
		}
		if(index == -1) {
			pBibitemInfo[nLastBibitem].nNextBibitem = i;
			pBibitemInfo[i].nLastBibitem = nLastBibitem;
			pBibitemInfo[i].nNextBibitem = -1;
			nLastBibitem = i;
		}
	}

	//generate re-sorted bbl file
	rewind(Fin);
	while(ftell(Fin) < pBibitemInfo[0].nStartPos) {
		if(fgets(b_in, nBibFileSize, Fin) == NULL) break;
		fputs(b_in, Fout);
	}
	index = nFirstBibitem;
	for(; index != -1; index = pBibitemInfo[index].nNextBibitem) {
		fseek(Fin, pBibitemInfo[index].nStartPos, SEEK_SET);
		for(i = 0; i < pBibitemInfo[index].nLine; i++) {
			if(fgets(b_in, nBibFileSize, Fin) == NULL) break;
			fputs(b_in, Fout);
		}
	}
	for(index = nFirstEnglishBibitem; index < nBibitems; index++) {
		if(pBibitemInfo[index].strAuthors[0] == Before_NonChineseName) {
			fseek(Fin, pBibitemInfo[index].nStartPos, SEEK_SET);
			for(i = 0; i < pBibitemInfo[index].nLine; i++) {
				if(fgets(b_in, nBibFileSize, Fin) == NULL) break;
				fputs(b_in, Fout);
			}
		}
	}
	fprintf(Fout, "%s\n", END_THEBIBLIOGRAPHY);
	
	fclose(Fin);
	fclose(Fout);
	sprintf(bakname, "%s.bak",inname);
	remove(bakname);
	rename(inname,bakname);
	rename(outname,inname);

	Fin = fopen(inname, "r");
	if(!Fin) {
		fprintf(stderr, "Failed to open %s for reading!\n", inname);
		freeall(); return -1;
	}
	Fout = fopen(outname, "wt");
	if(!Fout) {
		fprintf(stderr, "Cannot open %s for writing!\n", outname);
		freeall(); fclose(Fin); return -1;
	}
	return 0;
}
*/